@@ -24,9 +24,11 @@ struct PipeWireEncodedStreamPrivate {
     uint m_nodeId = 0;
     std::optional<uint> m_fd;
     Fraction m_maxFramerate;
+    int m_maxPendingFrames = 50;
     bool m_active = false;
     PipeWireBaseEncodedStream::Encoder m_encoder;
     std::optional<quint8> m_quality;
+    PipeWireBaseEncodedStream::EncodingPreference m_encodingPreference;
 
     std::unique_ptr<QThread> m_produceThread;
     std::unique_ptr<PipeWireProduce> m_produce;
@@ -107,6 +109,11 @@ void PipeWireBaseEncodedStream::setMaxFramerate(const Fraction &framerate)
         return;
     }
     d->m_maxFramerate = framerate;
+
+    if (d->m_produce) {
+        d->m_produce->setMaxFramerate(d->m_maxFramerate);
+    }
+
     Q_EMIT maxFramerateChanged();
 }
 
@@ -115,6 +122,23 @@ void PipeWireBaseEncodedStream::setMaxFramerate(quint32 numerator, quint32 denom
     setMaxFramerate({numerator, denominator});
 }
 
+void PipeWireBaseEncodedStream::setMaxPendingFrames(int maxPendingFrames)
+{
+    if (d->m_maxPendingFrames == maxPendingFrames) {
+        return;
+    }
+    if (d->m_produce) {
+        d->m_produce->setMaxPendingFrames(maxPendingFrames);
+    }
+    d->m_maxPendingFrames = maxPendingFrames;
+    Q_EMIT maxPendingFramesChanged();
+}
+
+int PipeWireBaseEncodedStream::maxBufferSize() const
+{
+    return d->m_maxPendingFrames;
+}
+
 void PipeWireBaseEncodedStream::setActive(bool active)
 {
     if (d->m_active == active)
@@ -153,6 +177,8 @@ void PipeWireBaseEncodedStream::refresh()
         d->m_produceThread->setObjectName("PipeWireProduce::input");
         d->m_produce = makeProduce();
         d->m_produce->setQuality(d->m_quality);
+        d->m_produce->setMaxPendingFrames(d->m_maxPendingFrames);
+        d->m_produce->setEncodingPreference(d->m_encodingPreference);
         d->m_produce->moveToThread(d->m_produceThread.get());
         d->m_produceThread->start();
         QMetaObject::invokeMethod(d->m_produce.get(), &PipeWireProduce::initialize, Qt::QueuedConnection);
@@ -177,7 +203,7 @@ PipeWireBaseEncodedStream::Encoder PipeWireBaseEncodedStream::encoder() const
 
 QList<PipeWireBaseEncodedStream::Encoder> PipeWireBaseEncodedStream::suggestedEncoders() const
 {
-    VaapiUtils vaapi;
+    auto vaapi = VaapiUtils::instance();
 
     QList<PipeWireBaseEncodedStream::Encoder> ret = {PipeWireBaseEncodedStream::VP8,
                                                      PipeWireBaseEncodedStream::VP9,
@@ -186,7 +212,7 @@ QList<PipeWireBaseEncodedStream::Encoder> PipeWireBaseEncodedStream::suggestedEn
     auto removeUnavailableEncoders = [&vaapi](const PipeWireBaseEncodedStream::Encoder &encoder) {
         switch (encoder) {
         case PipeWireBaseEncodedStream::VP8:
-            if (vaapi.supportsProfile(VAProfileVP8Version0_3) && avcodec_find_encoder_by_name("vp8_vaapi")) {
+            if (vaapi->supportsProfile(VAProfileVP8Version0_3) && avcodec_find_encoder_by_name("vp8_vaapi")) {
                 return false;
             } else {
                 return !avcodec_find_encoder_by_name("libvpx");
@@ -195,7 +221,7 @@ QList<PipeWireBaseEncodedStream::Encoder> PipeWireBaseEncodedStream::suggestedEn
             return !avcodec_find_encoder_by_name("libvpx-vp9");
         case PipeWireBaseEncodedStream::H264Main:
         case PipeWireBaseEncodedStream::H264Baseline:
-            if (vaapi.supportsProfile(encoder == PipeWireBaseEncodedStream::H264Main ? VAProfileH264Main : VAProfileH264ConstrainedBaseline)
+            if (vaapi->supportsProfile(encoder == PipeWireBaseEncodedStream::H264Main ? VAProfileH264Main : VAProfileH264ConstrainedBaseline)
                 && avcodec_find_encoder_by_name("h264_vaapi")) {
                 return false;
             } else {
@@ -209,6 +235,19 @@ QList<PipeWireBaseEncodedStream::Encoder> PipeWireBaseEncodedStream::suggestedEn
     return ret;
 }
 
+void PipeWireBaseEncodedStream::setEncodingPreference(PipeWireBaseEncodedStream::EncodingPreference preference)
+{
+    d->m_encodingPreference = preference;
+    if (d->m_produce) {
+        d->m_produce->setEncodingPreference(d->m_encodingPreference);
+    }
+}
+
+PipeWireBaseEncodedStream::EncodingPreference PipeWireBaseEncodedStream::encodingPreference()
+{
+    return d->m_encodingPreference;
+}
+
 bool PipeWireBaseEncodedStream::isActive() const
 {
     return d->m_active;
